package de.intevation.gnv.artifacts.cache;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;

import org.apache.log4j.Logger;

/**
 * The <code>CacheFactory</code> is used to initialize and retrieve Cache.
 * 
 * @author <a href="mailto:tim.englich@intevation.de">Tim Englich</a>
 */
public class CacheFactory {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(CacheFactory.class);

    private final static String CACHENAME = "artifactdata";

    /**
     * The singleton Instance of this Factory.
     */
    private static CacheFactory instance = null;

    /**
     *
     */
    private CacheManager cacheManager = null;

    /**
     * Basic-Constructor of this Class
     */
    private CacheFactory() {
        super();
    }

    /**
     * This Method provides an singleton Instance of this Class.
     *
     * @return an singleton Instance of this Class
     */
    public static CacheFactory getInstance() {
        if (instance == null) {
            instance = new CacheFactory();
        }
        return instance;
    }

    /**
     * Getting the ConnectionPool
     *
     * @return the ConnectionPool
     */
    public Cache getCache() {
        return this.cacheManager.getCache(CACHENAME);
    }

    /**
     * Initializes the ConnectionPool. Should only be called once on system
     * startup
     *
     * @param configurationFileName
     */
    public void initializeCache(String configurationFileName) {
        if (cacheManager == null) {
            cacheManager = new CacheManager(configurationFileName);
            cacheManager.addCache(CACHENAME);
        }
    }

    /**
     * Checks if the ConnectionPool has already been initialized.
     *
     * @return true if the ConnectionPool is initialized.
     */
    public boolean isInitialized() {
        return this.cacheManager != null;
    }

}
