package de.intevation.gnv.raster;

import de.intevation.gnv.jfreechart.CompactXYItems;
import de.intevation.gnv.jfreechart.PolygonDataset;
import de.intevation.gnv.jfreechart.PolygonSeries;

import de.intevation.gnv.raster.Vectorizer.Edge;

import gnu.trove.TDoubleArrayList;

import java.util.HashMap;
import java.util.List;

/**
 * @author <a href="mailto:sascha.teichmann@intevation.de">Sascha L. Teichmann</a>
 */
public class PolygonDatasetProducer
extends      AbstractProducer
{
    protected HashMap<Integer, PolygonSeries> polygonSeries;

    protected PolygonDatasetProducer() {
    }

    public PolygonDatasetProducer(
        double minX, double minY,
        double maxX, double maxY

    ) {
        super(minX, minY, maxX, maxY);
        polygonSeries = new HashMap<Integer, PolygonSeries>();
    }

    public void handleRings(
        List<Edge> rings,
        int        value,
        int        width,
        int        height
    ) {
        if (value == -1) {
            return;
        }

        Integer v = Integer.valueOf(value);

        PolygonSeries ps = polygonSeries.get(v);

        if (ps == null) {
            polygonSeries.put(v, ps = new PolygonSeries());
            ps.setAttribute("fill", v);
        }

        TDoubleArrayList vertices = new TDoubleArrayList();

        /* minX = 0*m1 + b1 <=> b1 = minX
         * maxX = (width-1)*m1 + b1
         * m1 = (maxX - minX)/(width-1)
         */

        double b1 = minX;
        double m1 = width != 1
           ? (maxX - minX)/(width-1)
           : 0d;

        double b2 = minY;
        double m2 = height != 1
           ? (maxY - minY)/(height-1)
           : 0d;

        for (Edge head: rings) {
            Edge current = head;
            do {
                vertices.add(m1*(current.a % width) + b1);
                vertices.add(m2*(current.a / width) + b2);
            }
            while ((current = current.next) != head);
            ps.addRing(new CompactXYItems(vertices.toNativeArray()));
            vertices.reset();
        }
    }

    public PolygonDataset getPolygonDataset() {
        return new PolygonDataset(polygonSeries.values());
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
